package at.jku.ssw.pi.map.ue09;

import java.util.ArrayList;

import at.jku.ssw.pi.Company;
import at.jku.ssw.pi.In;
import at.jku.ssw.pi.Out;
import at.jku.ssw.pi.map.Map;

class HashMapTest {

	// TODO: Change to the version for your operating system
	static final String FILENAME = "Top500.MacLinux.txt";
	// static final String FILENAME = "Top500.Windows.txt";

	static final boolean DEBUG_OUTPUT = false;

	public static void main(String[] args) {
		test1(new HashMapSeparateChaining());
		test2(new HashMapSeparateChaining());
		test3(new HashMapSeparateChaining());
		if (DEBUG_OUTPUT) {
			test4();
		}
		test5(new HashMapSeparateChaining());

		Out.println();
		Out.println("QUADRATIC");
		Out.println();
		test1(new HashMapQuadraticProbing());
		test2(new HashMapQuadraticProbing());
		test3(new HashMapQuadraticProbing());
		if (DEBUG_OUTPUT) {
			test4();
		}
		test5(new HashMapQuadraticProbing());
	}

	static void test1(Map m) {
		Out.println("\n*** Test 1 *** \n");

		m.put("JKU", "0732/2468");
		m.put("Buergerservice Linz", "0732/7070");
		m.put("Linz AG", "0732/3400-4000");
		m.put("OEBB", "05-1717");

		makeDot("test1a.dot", m);

		m.put("JKU", "0732/2468-1111");
		m.put("JKU", "0732/2468-2222");
		m.put("JKU", "0732/2468-3333");
		m.put("JKU", "0732/2468-4444");
		Out.println("m.get(\"JKU\")=" + m.get("JKU")); // "0732/2468-4444"

		Out.println("m.containsKey(\"Linz AG\")=" + m.containsKey("Linz AG")); // true
		Out.println("m.containsKey(\"Energie AG\")=" + m.containsKey("Energie AG")); // false

		m.put("Energie AG", null);
		Out.println("m.containsKey(\"Energie AG\")=" + m.containsKey("Energie AG")); // true
		Out.println("m.get(\"Energie AG\")=" + m.get("Energie AG")); // null

		Out.println("m.remove(\"JKU\")=" + m.remove("JKU"));
		Out.println("m.containsKey(\"JKU\")=" + m.containsKey("JKU")); // false

		makeDot("test1b.dot", m);
	}

	public static void makeDot(String filename, Map map) {
		Out.open(filename);
		Out.print(map.makeDot());
		Out.close();
	}

	public static void makeDot(String filename, Map map, float width) {
		Out.open(filename);
		Out.print(map.makeDot(width));
		Out.close();
	}

	static void test2(Map m) {
		Out.println("\n*** Test 2 *** \n");

		m.put("JKU", "0732/2468");
		m.put("Buergerservice Linz", "0732/7070");
		m.put("Linz AG", "0732/3400-4000");
		m.put("OEBB", "05-1717");

		Out.println("m.remove(\"Linz AG\")=" + m.remove("Linz AG"));

		m.put("Energie AG", "0800-81 8000");
		m.put("Sparkasse", "50-100-10100");

		Out.println("m.get(\"OEBB\")=" + m.get("OEBB")); // "05-1717"

		makeDot("test2.dot", m, 1.6f);
	}

	static void test3(Map m) {
		Out.println("\n*** Test 3 *** \n");
		Company[] companies = readCompanies(FILENAME);
		for (Company c : companies) {
			m.put(c.name, c);
		}

		makeDot("test3.dot", m, 3f);

		String[] names = { "Volkswagen", "Total", "Daimler", "Gazprom", "Panasonic", "Sony", "BMW", "Apple",
				"Coca-Cola", "Landesbank Baden-Wuerttemberg", "Woolworths", "Royal Ahold", "Bertelsmann", "Henkel" };

		for (int i = 0; i < names.length; i++) {
			String name = names[i];
			Out.print(HashMap.hash(name) + "  :  ");
			Company c = (Company) m.get(name);
			Out.println(c.toString());
		}
	}

	static void test4() {
		Out.println("\n*** Test 4 *** \n");
		Company[] companies = readCompanies(FILENAME);
		for (int i = 0; i < companies.length; i++) {
			Company c = companies[i];
			System.out.println(String.format("%s: %d", c.name, c.name.hashCode()));
		}
	}

	static void test5(Map m) {
		String[] keys = new String[26];
		for (char c = 'A'; c <= 'Z'; c++) {
			keys[c - 'A'] = Character.toString(c);
			m.put(keys[c - 'A'], null);
		}

		for (char c = 'A'; c <= 'Z'; c++) {
			Out.println("m.containsKey(\"" + keys[c - 'A'] + "\")=" + m.containsKey(keys[c - 'A']));
		}
	}

	static Company[] readCompanies(String filename) {
		In.open(filename);
		ArrayList<Company> result = new ArrayList<>();
		do {
			Company company = Company.readCompany();
			if (company != null) {
				result.add(company);
			}
		} while (In.done());
		In.close();
		return result.toArray(new Company[0]);
	}
}
